#include "NFC.h"
#include <SPI.h>
#include <PubSubClient.h>
#include <MFRC522.h>

NFC::NFC(int pin_ss, int pin_rst)
{
    SPI.begin();

    rfid = new MFRC522(pin_ss, pin_rst);
    rfid->PCD_Init();
}

bool NFC::hasNewCard()
{
    if(rfid->PICC_IsNewCardPresent() && rfid->PICC_ReadCardSerial())
    {
        byte atqaLen = sizeof(atqa);
        MFRC522::StatusCode status = rfid->PICC_RequestA(atqa, &atqaLen);
        Serial.printf("PICC_RequestA: 0x%02x\n", status);
        if(status == MFRC522::STATUS_OK && atqa[0] == 0x44)
        {
            return true;
        }
    }
    return false;
}

bool NFC::selectCard()
{
    MFRC522::StatusCode status = rfid->PICC_Select(&(rfid->uid));
    Serial.printf("PICC_Select: 0x%02x\n", status);
    if(status != MFRC522::STATUS_OK)
    {
        return false;
    }

    cardSelected = true;
    uid = rfid->uid;
    Serial.println("Card Selected");
    return true;
}

bool NFC::deselectCard()
{
    MFRC522::StatusCode status = rfid->PICC_HaltA();
    Serial.printf("PICC_HaltA: 0x%02x\n", status);
    if(status != MFRC522::STATUS_OK)
    {
        return false;
    }

    cardSelected = false;
    Serial.println("Card Deselected");

    return true;
}

bool NFC::isCardLost()
{
    uint8_t control = 0x00;
    for(int i=0; i<3; i++)
    {
        if(!rfid->PICC_IsNewCardPresent())
        {
            if(rfid->PICC_ReadCardSerial())
            {
                control |= 0x16;
            }
            if(rfid->PICC_ReadCardSerial())
            {
                control |= 0x16;
            }
            control += 0x1;
        }
        control += 0x4;
    }
    return !(control == 13 || control == 14);
}

MFRC522::StatusCode NFC::Transceive(byte* command, byte command_len, byte* response, byte response_len)
{
    return rfid->PCD_TransceiveData(command, command_len, response, &response_len, NULL, 0, true);
}

MFRC522::Uid NFC::getUID()
{
    return uid;
}